package com.loafle.overflow.central.commons.utils;

import io.grpc.Context;
import io.grpc.Metadata;

import static io.grpc.Metadata.ASCII_STRING_MARSHALLER;

public class SessionMetadata {
  /*
   * digits: 0-9 uppercase letters: A-Z (normalized to lower) lowercase letters:
   * a-z special characters: -_.
   */

  public static final String CLIENT_TYPE_KEY = "OVERFLOW_GRPC_CLIENT_TYPE";
  public static final String SESSION_ID_KEY = "OVERFLOW_GRPC_SESSION_ID";
  public static final String TARGET_ID_KEY = "OVERFLOW_GRPC_TARGET_ID";

  public static final Context.Key<String> CTX_CLIENT_TYPE_KEY = Context.key(CLIENT_TYPE_KEY);
  public static final Context.Key<String> CTX_SESSION_ID_KEY = Context.key(SESSION_ID_KEY);
  public static final Context.Key<String> CTX_TARGET_ID_KEY = Context.key(TARGET_ID_KEY);

  public static final Metadata.Key<String> METADATA_CLIENT_TYPE_KEY = Metadata.Key.of(CLIENT_TYPE_KEY,
      ASCII_STRING_MARSHALLER);
  public static final Metadata.Key<String> METADATA_SESSION_ID_KEY = Metadata.Key.of(SESSION_ID_KEY,
      ASCII_STRING_MARSHALLER);
  public static final Metadata.Key<String> METADATA_TARGET_ID_KEY = Metadata.Key.of(TARGET_ID_KEY,
      ASCII_STRING_MARSHALLER);

  public static ClientType getClientType() {
    return ClientType.valueOf(CTX_CLIENT_TYPE_KEY.get());
  }

  public static String getSessionID() {
    return CTX_SESSION_ID_KEY.get();
  }

  public static String getTargetID() {
    return CTX_TARGET_ID_KEY.get();
  }

  public static enum ClientType {
    MEMBER("MEMBER"), PROBE("PROBE");

    final private String name;

    private ClientType(String name) {
      this.name = name;
    }

    public String toString() {
      return name;
    }
  }
}
